/*
 * WizardPanelMaker.m
 *
 * Implementation of the WizardPanelMaker class for the
 * WizardPalette palette
 *
 * Copyright (c) 2006, by Saso Kiselkov
 *
 * For license details please see the file COPYING included with this
 * source distribution package.
 */

#import "WizardPanelMaker.h"

#import <WizardKit/WizardKit.h>

#import <Foundation/NSBundle.h>

#import "GormWizardPanel.h"

@implementation WizardPanelMaker

- (void) encodeWithCoder: (NSCoder *) aCoder
{
}

- initWithCoder: (NSCoder *) aCoder
{
  GormWizardPanel * wizardPanel;

  wizardPanel = [[GormWizardPanel alloc]
    initWithContentRect: NSMakeRect(400, 400, 500, 300)
              styleMask: NSTitledWindowMask | NSClosableWindowMask |
                         NSResizableWindowMask
                backing: NSBackingStoreRetained
                  defer: YES];

  [wizardPanel setTitle: _(@"Wizard Panel")];

  [self release];

  return wizardPanel;
}

@end
