/*
 * WizardPalette.m
 *
 * Implementation of the WizardPalette class for the
 * WizardPalette palette
 *
 * Copyright (c) 2006, by Saso Kiselkov
 *
 * For license details please see the file COPYING included with this
 * source distribution package.
 */

#import "WizardPalette.h"

#import "WizardPanelMaker.h"

@implementation WizardPalette

- (void) finishInstantiate
{
  [self associateObject: [[WizardPanelMaker new] autorelease]
                   type: IBWindowPboardType
                   with: wizardPanelView];
}

@end
