/*
 * WizardKit.h
 *
 * Master header file for the WizardKit framework
 *
 * Copyright (c) 2006, by Saso Kiselkov
 *
 * For license details please see the file COPYING included with this
 * source distribution package.
 */

#ifndef _WIZARD_KIT_H_
#define _WIZARD_KIT_H_

#import <WizardKit/WKWizardPanel.h>

#endif // _WIZARD_KIT_H_
