/*
 * WizardPanelMaker.h
 *
 * Interface declaration of the WizardPanelMaker class for the
 * WizardPalette palette
 *
 * Copyright (c) 2006, by Saso Kiselkov
 *
 * For license details please see the file COPYING included with this
 * source distribution package.
 */

#import <Foundation/NSObject.h>
#import <InterfaceBuilder/InterfaceBuilder.h>

@interface WizardPanelMaker : NSObject <NSCoding>

@end
